import React, { useState } from "react";
import {
	Box,
	Button,
	Typography,
	Modal,
	InputAdornment,
	styled,
} from "@mui/material";
import CloseIcon from "@mui/icons-material/Close";
import Visibility from "@mui/icons-material/Visibility";
import VisibilityOff from "@mui/icons-material/VisibilityOff";
import { useForm, Controller } from "react-hook-form";
import { CustomIconButton, CustomTextField } from "../AddUserModal/styled.css";

interface FormData {
	name: string;
	email: string;
	password: string;
	registeredAddress: string;
	phone: string; // Added phone field
}

interface AddAdminModalProps {
	open: boolean;
	handleClose: () => void;
	handleCreate: (data: FormData) => void;
}

const AddAdminModal: React.FC<AddAdminModalProps> = ({
	open,
	handleClose,
	handleCreate,
}) => {
	const [showPassword, setShowPassword] = useState(false);

	const {
		control,
		handleSubmit,
		formState: { errors },
		reset,
	} = useForm<FormData>({
		defaultValues: {
			name: "",
			email: "",
			password: "",
			registeredAddress: "",
			phone: "", // Initialize phone field
		},
	});

	const onSubmit = (data: FormData) => {
		handleCreate(data);
		handleClose();
		reset();
	};

	const togglePasswordVisibility = () => {
		setShowPassword((prev) => !prev);
	};

	return (
		<Modal
			open={open}
			onClose={(e, reason) => {
				if (reason === "backdropClick") {
					return;
				}
				handleClose();
			}}
			aria-labelledby="add-admin-modal"
		>
			<Box
				sx={{
					position: "absolute",
					top: "50%",
					left: "50%",
					transform: "translate(-50%, -50%)",
					width: 600,
					bgcolor: "background.paper",
					boxShadow: 24,
					p: 3,
					borderRadius: 2,
				}}
			>
				{/* Header */}
				<Box
					sx={{
						display: "flex",
						justifyContent: "space-between",
						alignItems: "center",
					}}
				>
					<Typography variant="h6" fontWeight={600}>
						Add Admin
					</Typography>
					<CustomIconButton onClick={handleClose}>
						<CloseIcon />
					</CustomIconButton>
				</Box>

				{/* Horizontal Line */}
				<Box sx={{ borderBottom: "1px solid #ddd", my: 2 }} />

				{/* Form */}
				<form onSubmit={handleSubmit(onSubmit)}>
					{/* First Row - Admin Name & Email */}
					<Box sx={{ display: "flex", gap: 2, mb: 2 }}>
						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								flex: 1,
							}}
						>
							<Typography variant="body2" fontWeight={500}>
								Admin Name
							</Typography>
							<Controller
								name="name"
								control={control}
								rules={{
									required: "Admin Name is required",
									minLength: {
										value: 3,
										message:
											"Minimum 3 characters required",
									},
									maxLength: {
										value: 30,
										message:
											"Maximum 30 characters allowed",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										required
										placeholder="Enter Admin Name"
										fullWidth
										size="small"
										error={!!errors.name}
										helperText={errors.name?.message}
									/>
								)}
							/>
						</Box>

						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								flex: 1,
							}}
						>
							<Typography variant="body2" fontWeight={500}>
								Email
							</Typography>
							<Controller
								name="email"
								control={control}
								rules={{
									required: "Email is required",
									pattern: {
										value: /\S+@\S+\.\S+/,
										message:
											"Please enter a valid email address.",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										error={!!errors.email}
										helperText={errors.email?.message}
										type="email"
										placeholder="Email"
										required
										fullWidth
									/>
								)}
							/>
						</Box>
					</Box>

					{/* Second Row - Password, Phone, Address */}
					<Box sx={{ display: "flex", gap: 2 }}>
						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								flex: 1,
							}}
						>
							<Typography variant="body2" fontWeight={500}>
								Password
							</Typography>
							<Controller
								name="password"
								control={control}
								rules={{
									required: "Password is required",
									minLength: {
										value: 6,
										message:
											"Password must be at least 6 characters",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										required
										placeholder="Enter Password"
										type={
											showPassword ? "text" : "password"
										}
										fullWidth
										size="small"
										error={!!errors.password}
										helperText={errors.password?.message}
										slotProps={{
											input: {
												endAdornment: (
													<InputAdornment position="end">
														<CustomIconButton
															onClick={
																togglePasswordVisibility
															}
															edge="end"
														>
															{showPassword ? (
																<VisibilityOff />
															) : (
																<Visibility />
															)}
														</CustomIconButton>
													</InputAdornment>
												),
											},
										}}
									/>
								)}
							/>
						</Box>

						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								flex: 1,
							}}
						>
							<Typography variant="body2" fontWeight={500}>
								Phone
							</Typography>
							<Controller
								name="phone"
								control={control}
								rules={{
									required: "Phone number is required",
									pattern: {
										value: /^[0-9]{10}$/,
										message:
											"Enter a valid 10-digit phone number",
									},
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										required
										placeholder="Enter Phone Number"
										fullWidth
										size="small"
										error={!!errors.phone}
										helperText={errors.phone?.message}
									/>
								)}
							/>
						</Box>

						<Box
							sx={{
								display: "flex",
								flexDirection: "column",
								flex: 1,
							}}
						>
							<Typography variant="body2" fontWeight={500}>
								Address
							</Typography>
							<Controller
								name="registeredAddress"
								control={control}
								rules={{
									required: "Address is required",
								}}
								render={({ field }) => (
									<CustomTextField
										{...field}
										required
										placeholder="Enter Address"
										fullWidth
										size="small"
										error={!!errors.registeredAddress}
										helperText={
											errors.registeredAddress?.message
										}
									/>
								)}
							/>
						</Box>
					</Box>

					{/* Submit Button */}
					<Box
						sx={{
							display: "flex",
							justifyContent: "flex-end",
							mt: 3,
						}}
					>
						<Button
							type="submit"
							sx={{
								backgroundColor: "#52ACDF",
								color: "white",
								borderRadius: "8px",
								width: "117px",
								"&:hover": { backgroundColor: "#439BC1" },
							}}
						>
							Add Admin
						</Button>
					</Box>
				</form>
			</Box>
		</Modal>
	);
};

export default AddAdminModal;
